
#include <at89c51RC2.h>
#include "inc\main.h"
#include "inc\definitions.h"
#include "inc\RS232.h"


// GLOBAL VARIABLES

// Timer 1 counter
unsigned char time_counter;

// Baudrate
unsigned char baudrate;

// Eingangspuffer
unsigned char data input_buffer[MAX_INPUT];
bit new_input_data;
unsigned char input_len;




void main (void)
{
//	int i;
	EnterBootLoader();
	init();
	while (1)
	{
		ES = 0;					// lock serial interrupt during input buffer analysis
		if (new_input_data)		// check flag if new data is in input buffer
		{
			new_input_data = 0;	// reset flag
			if((input_len>=MAX_INPUT)&&(input_buffer[MAX_INPUT-1]==13))	// check length of input data buffer
			{
				input_buffer[MAX_INPUT-1] = 0;		// clear last char in buffer, this replaces the <CR> with the termination zero -> more practical for string processing
				//send_error_message(ERROR_NUMBER_BUF_OVERFLOW);	// generate a overflow error message
				input_len=0;						// reset length of input buffer->clear input buffer
			}
			else
			{
				if(input_buffer[input_len-1]==13)	//	check if <CR> was received as last
				{
					input_buffer[input_len-1]=0;	//	replace the <CR> with a terminating zero which makes it easier in further string processing
					//analyse_input_buffer();			//	call the command processing function
					send_string_with_cr(input_buffer);
				}
			}
		}
		ES = 1;	// unlock the serial interrupt again
		//send_char('A');
	}

}

void EnterBootLoader(void)
{	
	int i;				   
	bit ea_save;
	ea_save = EA;
	EA = 0;
	DPH = 0x00;
	DPL = 0x00;
	ACC = 0x04;
	ASM_MOV_R1_A();
	ACC = 0x00;
	MAP_BOOT;
	__API_FLASH_ENTRY_POINT();
	UNMAP_BOOT;
	EA = ea_save; // restore interrupt state
	for(i=0;i<=2000;i++)
	{;}
//	send_string("Programmiermodus aktiv");
//	send_char(0x0D);
	for(i=0;i<=2000;i++)
	{;}
}


//////////////////////////////////////////////////////
//	Name:	timer1_interrupt						//
//--------------------------------------------------//
//	Parameter:										//
//	none											//
//--------------------------------------------------//
// Timer 1 Interrupt Routine						//
// Wird zur Read-Timeout Erkennung, Torzeit 		//
// und fr das Blinken der OP-LED verwendet			//
// hierfr wird die Routine normalerweise 			//
// alle 50ms aufgerufen								//
//////////////////////////////////////////////////////
void timer1_interrupt(void)	interrupt 3 using 2 
{
	TF1 = 0;				// reset timer 1 interrupt-flag
	time_counter++;			// Zhler fr Timeout und OP-Signal
	if (time_counter>=7)	// 500ms sind verstrichen ?
	{
		time_counter=0;		// Timeout Zhler rcksetzen
		LED5 = ~LED5;	// OP-Signal umschalten
	}
/*
	if (time_counter%2 == 0)
	{
		Relais1
	}		
	else
	{
		Relais2
	} */
}





void init(void)
{
	EX0 = 0;	// disable external interrupt0 for digital in detection
	EA = 0;			// disable interrupts

	Relais0
	baudrate = 2;
	time_counter=0;
	input_len=0;
	new_input_data=0;


	init_communication();

	ET1 = 0;		// disable timer1 interrupt
	TR1 = 0;		// disable timer1
	TF1 = 0;		// reset timer 1 interrupt-flag
	TL1 = 0;		//	reset timer value
	TH1 = 0;		//
	TMOD = 0x10;	// clear timer1 mode bits
	ET1	= 1;		// enable timer1 overflow interrupt
	TR1 = 1;		// enable timer1 again
	EX0 = 0;	// disable external interrupt0 for digital in detection

	
	EA = 1;		// enable interrupts
}




