
#include <at89c51RC2.h>
#include "inc\RS232.h"
#include "inc\Definitions.h"

extern unsigned char baudrate;
extern unsigned char data input_buffer[MAX_INPUT];
extern bit new_input_data;
extern unsigned char input_len;


void init_communication(void)
{
	ES = 0;				// Disable serial interrupt
	SCON = 0x52;		// Set to 8-Bit UART
	T2CON  = 0x30;		// set timer 2 control register for use as baudrate generator
	RCAP2H = 0xFF;		// preset the timer 2 register for correct rollover
	switch(baudrate)
	{
		case 0:
				{
					RCAP2L = 0xB8;	//4800 RCAP2 = 0xFF98
					break;
				}
		case 1:
				{
					RCAP2L = 0xDC;	//9600 RCAP2 = 0xFFCC
					break;
				}
		case 2:
				{
					RCAP2L = 0xEE;	//19200 RCAP2 = 0xFFE6
					break;
				}
		case 3:
				{
					RCAP2L = 0xF7;	//38400 RCAP2 = 0xFFF3
					break;
				}
		default:
				{
					RCAP2L = 0xEE;	//19200
					break;
				}
	}
	TR2 = 1;			// start timer 2
	IPL0 = 0x10;//&=0xEF
	IPH0 |= 0x18;//|=0x10		// Serieller Interrupt hat hohe priority
	RI = 0;
	TI = 0;
	ES = 1;
}


//////////////////////////////////////////////////////
//	Name:	send_char								//
//--------------------------------------------------//
//	Parameter:										//
//	char Text	-	The char which is to output		//
//--------------------------------------------------//
//	Purpose:										//
//	Routine to write a single char					//
//	to the serial port								//
//////////////////////////////////////////////////////
void send_char(char single_char)	//reentrant	//reentrant because could be executed from interrupt
{
//	while (!TI);			//	Wait until Char has send	     
//	TI=0;               
	SBUF = single_char;		//	Write Char to serial Buffer
	while (!TI);			//	Wait until Char has send	     
	TI=0;               
}


//////////////////////////////////////////////////////
//	Name:	send_string_with_cr						//
//--------------------------------------------------//
//	Parameter:										//
//	char* Text	-	The string which is to output	//
//--------------------------------------------------//
//	Purpose:										//
//	Routine to write a zero terminated char_string	//
//	to the serial port								//
//////////////////////////////////////////////////////
void send_string_with_cr(char* string)
{
	send_string(string);
	send_char(0x0D);
}

//////////////////////////////////////////////////////
//	Name:	send_string							//
//--------------------------------------------------//
//	Parameter:										//
//	char* Text	-	The string which is to output	//
//--------------------------------------------------//
//	Purpose:										//
//	Routine to write a zero terminated char_string	//
//	to the serial port								//
//////////////////////////////////////////////////////
void send_string(char* string) //	reentrant	//reentrant because could be executed from interrupt
{
	unsigned char i=0;

	while (string[i]!=0)			//	Countinue until zero termination
	{
		send_char(string[i]);
		i++;					//	next Character
	}
}

//////////////////////////////////////////////////////
//	Name:	Serial_Interrupt0						//
//--------------------------------------------------//
//	Parameter:										//
//	none											//
//--------------------------------------------------//
//	Purpose:										//
//	A Routine to handle Serial Interrupt			//
//	Supports serial ASB Commands					//
//////////////////////////////////////////////////////
void Serial_Interrupt0(void) interrupt 4 
{
	char c = 0;
	
	if (RI != 0)
	{
		c = SBUF;							//	Lese das Zeichen aus dem Schnittstellepuffer
		RI=0; 								//  Clear the existing interrupt request
		new_input_data=1;					//	new_input_data ist ein verlngerte Interrupt Flag
		if (input_len<MAX_INPUT)			//	Check if Input Buffer is Full		
			input_buffer[input_len] = c;	//	store the Char in the Input Buffer Array
		else
			input_buffer[MAX_INPUT-1] = c;	//	und speichere es am Ende des Einagebpuffers
		input_len++;					//	Next Array Position
	}
}